/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */
#include <zephyr.h>
#include <errno.h>
#include <aarch32/cortex_m/tz.h>
#include <sys/reboot.h>
#include <sys/util.h>
#include <autoconf.h>
#include <string.h>
#include <bl_validation.h>
#include <aarch32/cortex_m/cmse.h>

#include <random/rand32.h>


#if USE_PARTITION_MANAGER
#include <pm_config.h>
#endif
#ifdef CONFIG_SPM_SERVICE_FIND_FIRMWARE_INFO
#include <fw_info.h>
#endif

/*
 * Secure Entry functions to allow access to secure services from non-secure
 * firmware.
 *
 * Note: the function will be located in a Non-Secure
 * Callable region of the Secure Firmware Image.
 *
 * These should not be called directly. Instead call them through their wrapper
 * functions, e.g. call spm_request_read_nse() via spm_request_read().
 */
#ifdef CONFIG_SPM_SERVICE_RNG
#include "nrf_cc3xx_platform_ctr_drbg.h"

static nrf_cc3xx_platform_ctr_drbg_context_t ctr_drbg_ctx;
#endif /* CONFIG_SPM_SERVICE_RNG */

#ifdef CONFIG_SPM_SERVICE_RNG
#ifdef MBEDTLS_CONFIG_FILE
#include MBEDTLS_CONFIG_FILE
#else
#include "mbedtls/config.h"
#endif /* MBEDTLS_CONFIG_FILE */

#include <mbedtls/platform.h>
#include <mbedtls/entropy_poll.h>
#endif /* CONFIG_SPM_SERVICE_RNG */

#include "mbedtls/pk.h"
#include "mbedtls/entropy.h"
#include "mbedtls/ctr_drbg.h"

mbedtls_entropy_context entropy;
mbedtls_ctr_drbg_context ctr_drbg;


static bool ptr_in_secure_area(intptr_t ptr)
{
	return arm_cmse_addr_is_secure(ptr) == 1;
}

int spm_secure_services_init(void)
{
	int err = 0;

#ifdef CONFIG_SPM_SERVICE_RNG
	mbedtls_platform_context platform_ctx = {0};

	err = nrf_cc3xx_platform_ctr_drbg_init(&ctr_drbg_ctx, NULL, 0);
//	err = mbedtls_platform_setup(&platform_ctx);
        printk("er %d\n", err);
#endif
	return err;
}

//#ifdef CONFIG_SPM_SERVICE_READ
#include <mbedtls/md.h>
#include <mbedtls/aes.h>


static const unsigned char * const ENC_PUB_KEY= (unsigned char *)
"-----BEGIN PUBLIC KEY-----\n"
"MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA8Ftv7jOw1x/i9znh6Z6K\n"
"5+Bbr/Uty+EfQs1DmGozKDnxjaq75wwJQBar5/Q7nGw/6wsfsgcISOSX8iFwKm6/\n"
"uND1Lr00FtmdwIXEtpM0bNkRrWSv9sGS9LfLU+FyGepBD0kGgYdeuFQMNNipMf78\n"
"XMlK+glZHAoIKozIyblp3IlYfwZsjeH5Ad0W3iYPI18PmUkX4k7OWtn3Kq95DmHZ\n"
"fhR4yKjH6PrY2oA78vpHD9Gg8m0SVPy6JE4tjWuBxwIfdbr2OqQC3WfyRV0Gfd3E\n"
"VpZjCoC78k4jeLezXc5fvS7hHaTUymm3EK3DsyFqgviMdk26lQyn0RIEEuwGtKLV\n"
"7wIDAQAB\n"
"-----END PUBLIC KEY-----\n"
;

static const unsigned char * const DEC_PRIVATE_KEY_X= (unsigned char *)
"-----BEGIN RSA PRIVATE KEY-----\n"
"MIIEowIBAAKCAQEA8Ftv7jOw1x/i9znh6Z6K5+Bbr/Uty+EfQs1DmGozKDnxjaq7\n"
"5wwJQBar5/Q7nGw/6wsfsgcISOSX8iFwKm6/uND1Lr00FtmdwIXEtpM0bNkRrWSv\n"
"9sGS9LfLU+FyGepBD0kGgYdeuFQMNNipMf78XMlK+glZHAoIKozIyblp3IlYfwZs\n"
"jeH5Ad0W3iYPI18PmUkX4k7OWtn3Kq95DmHZfhR4yKjH6PrY2oA78vpHD9Gg8m0S\n"
"VPy6JE4tjWuBxwIfdbr2OqQC3WfyRV0Gfd3EVpZjCoC78k4jeLezXc5fvS7hHaTU\n"
"ymm3EK3DsyFqgviMdk26lQyn0RIEEuwGtKLV7wIDAQABAoIBAAVKzh0dkEA8WH//\n"
"C0Vck0xluT8Xk+mQTGLpKBrcA8JX/ESVRMvijPWXSjOyQfnAvGuNdCY0V29bcSkI\n"
"qXKVu6zzFOLxkYsLQZjymRE4DohzAxuhQuqY+oZe4aA4L434hXQztGv3shM27onf\n"
"V2JdbesRF5CH0il24xUkwmsGpRkQ9i/j79nu2N14na+yXhP8fGgdHzTHFNzY18Uf\n"
"kWXjDqMw9oJSYye45oh/CXVm3PAjEHC4owNw8uyV25zTYCCZLG2syGvsAIH9aNIk\n"
"OIORcRDTxgG4NYEmEZuLPqzhm1/ttmbZjNqjbg00wuzcLjW5Tj0cG424w7uKsxK/\n"
"30AuHWECgYEA/Pp/WLJp09zMTLydPWHZx2GVAnvaG9Z8hydDx1ryogaTUCNNBnxM\n"
"gqZ+X0JfVBPUUXTELihWr9g+Nu74U/++N1SLFw6ej1vOSEkkuDNduO8moXWC54j1\n"
"eugNzHlFQ+HrEW3AxzLaGH9xCilDR3iNicMjD7aZk2XucZfpOVO+63ECgYEA8zpZ\n"
"WiJPBa/kDgXZZxIkK4xpKRw/iec7zEWIslRwuGYDEyEHwcKJY/h5cXUd0OwFuUOi\n"
"MmXH3EDsn769hWwzlJjmaFxg6udFEunDGaNjuWTNz91V9jumlXlFoJ3gho+sbGqI\n"
"YrbpcWSQQr4ByJRZoZV6Yilvzkm1pb0EOLN/Z18CgYAkAsB37sWzu911GvBDGZe3\n"
"kwN0HA87ITgOwR8b1K1vNQJpSO0g/Daj7nFUvAz32e7CPZ3jeCiG1YBdr3y38FKy\n"
"a511Bca6X3YowgixE9bGiXx1RVYCAXP4L+ou7iBW/mTI0Ve8w1daX2HG8YK7PMTh\n"
"FF78UKDh44Oitd4kI/CscQKBgCiyvLy+PObAtbI6/FOoz3CSAA6m8WLL1E+r2uGs\n"
"C2hveVg8EKY1PPQDYZ7AcArDU3nafUtdiIGsQ2u/aMrofgEBTJxtyLHFnOM7RjAG\n"
"DOu6mE2NxZJ8DLW6IxJrGGSrqbhCYyvRW2Jj51ejn3zGhfDpqB7AvDqSTti6tfge\n"
"4NSVAoGBAIrGt/ikOjkOt4V0hM4lC8HwbEeIH4GDaybgYf9yCfr68PvGdOGQG3B7\n"
"dCvw+VIrrR2KsFxFSzIr1YhiL7L0DKs2OMVLXT/yW1JTcPdn/JNBq3XyDwjjMvS1\n"
"+MXPNxP/qqpo0ikn830PgCxRFcYDJZQ5cNbKTQm6O3bJY+foxqfL\n"
"-----END RSA PRIVATE KEY-----\n"
;

__TZ_NONSECURE_ENTRY_FUNC
void spm_init_nse()
{

}

__TZ_NONSECURE_ENTRY_FUNC
int spm_encrypt_nse(char *ciphered, char *data, size_t len)
{
    return 0;
}

__TZ_NONSECURE_ENTRY_FUNC
int spm_change_key_nse(int keyPurpose, unsigned char *new_key, int key_size)
{
    return 0;
}

__TZ_NONSECURE_ENTRY_FUNC
int spm_checksum_nse(int keyPurpose, void *blks, int block_count, uint8_t *digest)
{
    return 0;
}

static unsigned char result[512];
static char *pers = "mbedtls_pk_encrypt";

__TZ_NONSECURE_ENTRY_FUNC
int spm_encrypt_key_nse(int keyPurpose, char *output, int outputLen)
{
    int olen = -1;

    char data[] = "603DEB1015CA71BE2B73AEF0857D7781";

    mbedtls_pk_context pk;
    mbedtls_pk_init(&pk);

    mbedtls_ctr_drbg_init(&ctr_drbg);
    mbedtls_entropy_init(&entropy);

    int ret = mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func, &entropy,
                                (const unsigned char*) pers, strlen(pers));
    if (ret != 0) {
        goto exit;
    }

    if (mbedtls_pk_parse_public_key(&pk, ENC_PUB_KEY,
                                    strlen((const char *)ENC_PUB_KEY) + 1) != 0) {
        goto exit;
    }

    ret = mbedtls_pk_encrypt(&pk,
                           (const unsigned char *)data, sizeof(data),
                           (unsigned char *)output, &olen, outputLen,
                           mbedtls_ctr_drbg_random, &ctr_drbg);
    if (ret != 0) {
        olen = -1;
        goto exit;
    }

exit:
    mbedtls_pk_free(&pk);

    return olen;
}

__TZ_NONSECURE_ENTRY_FUNC
int spm_decrypt_key_nse(int keyPurpose, unsigned char *buf, int size)
{
    mbedtls_pk_context pk;
    mbedtls_pk_init(&pk);
    mbedtls_ctr_drbg_init(&ctr_drbg);
    mbedtls_entropy_init(&entropy);

    int ret = mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func, &entropy,
                                (const unsigned char*) pers, strlen(pers));
    if (ret != 0) {
        mbedtls_entropy_free(&entropy);
        return -1;
    }

    memset(result, 0, sizeof(result));
    ret = mbedtls_pk_parse_key(&pk, DEC_PRIVATE_KEY_X,
                               strlen((const char *)DEC_PRIVATE_KEY_X) + 1, NULL, 0);
    if (ret != 0) {
        mbedtls_pk_free(&pk);
        return -1;
    }

    size_t olen = 0;
    ret = mbedtls_pk_decrypt(&pk, buf, size, result, &olen, sizeof(result),
                             mbedtls_ctr_drbg_random, &ctr_drbg);
    if (ret != 0) {
        //printk("mbedtls_pk_decrypt: %d\n", ret);
        mbedtls_pk_free(&pk);
        return -1;
    }

    mbedtls_pk_free(&pk);

    return (int)olen;
}

//#endif /* CONFIG_SPM_SERVICE_READ */


#ifdef CONFIG_SPM_SERVICE_REBOOT
__TZ_NONSECURE_ENTRY_FUNC
void spm_request_system_reboot_nse(void)
{
	sys_reboot(SYS_REBOOT_COLD);
}
#endif /* CONFIG_SPM_SERVICE_REBOOT */


#ifdef CONFIG_SPM_SERVICE_RNG
__TZ_NONSECURE_ENTRY_FUNC
int spm_request_random_number_nse(uint8_t *output, size_t len, size_t *olen)
{
	int err = -EINVAL;

	if (ptr_in_secure_area((intptr_t)output) ||
	    ptr_in_secure_area((intptr_t)olen)) {
		return -EINVAL;
	}

//	if (len != MBEDTLS_ENTROPY_MAX_GATHER) {
//		return -EINVAL;
//	}

	err = nrf_cc3xx_platform_ctr_drbg_get(&ctr_drbg_ctx, output, len, olen);
	if (*olen != len) {
		return -EINVAL;
	}
// replaced by the previous block
//	err = mbedtls_hardware_poll(NULL, output, len, olen);
	return err;
}
#endif /* CONFIG_SPM_SERVICE_RNG */

#ifdef CONFIG_SPM_SERVICE_S0_ACTIVE
__TZ_NONSECURE_ENTRY_FUNC
int spm_s0_active(uint32_t s0_address, uint32_t s1_address, bool *s0_active)
{
	const struct fw_info *s0;
	const struct fw_info *s1;
	bool s0_valid;
	bool s1_valid;

	if (ptr_in_secure_area((intptr_t)s0_active)) {
		return -EINVAL;
	}

	s0 = fw_info_find(s0_address);
	s1 = fw_info_find(s1_address);

	s0_valid = (s0 != NULL) && (s0->valid == CONFIG_FW_INFO_VALID_VAL);
	s1_valid = (s1 != NULL) && (s1->valid == CONFIG_FW_INFO_VALID_VAL);

	if (!s1_valid && !s0_valid) {
		return -EINVAL;
	} else if (!s1_valid) {
		*s0_active = true;
	} else if (!s0_valid) {
		*s0_active = false;
	} else {
		*s0_active = s0->version >= s1->version;
	}

	return 0;
}
#endif /* CONFIG_SPM_SERVICE_S0_ACTIVE */

#ifdef CONFIG_SPM_SERVICE_FIND_FIRMWARE_INFO
__TZ_NONSECURE_ENTRY_FUNC
int spm_firmware_info_nse(uint32_t fw_address, struct fw_info *info)
{
	const struct fw_info *tmp_info;

	if (info == NULL) {
		return -EINVAL;
	}

	/* Ensure that fw_address is within secure area */
	if (!ptr_in_secure_area(fw_address)) {
		return -EINVAL;
	}

	/* Ensure that *info is in non-secure RAM */
	if (ptr_in_secure_area((intptr_t)info)) {
		return -EINVAL;
	}

	tmp_info = fw_info_find(fw_address);

	if (tmp_info != NULL) {
		memcpy(info, tmp_info, sizeof(*tmp_info));
		return 0;
	}

	return -EFAULT;
}
#endif /* CONFIG_SPM_SERVICE_FIND_FIRMWARE_INFO */


#ifdef CONFIG_SPM_SERVICE_PREVALIDATE
__TZ_NONSECURE_ENTRY_FUNC
int spm_prevalidate_b1_upgrade_nse(uint32_t dst_addr, uint32_t src_addr)
{
	if (!bl_validate_firmware_available()) {
		return -ENOTSUP;
	}
	bool result = bl_validate_firmware(dst_addr, src_addr);
	return result;
}
#endif /* CONFIG_SPM_SERVICE_PREVALIDATE */


#ifdef CONFIG_SPM_SERVICE_BUSY_WAIT
__TZ_NONSECURE_ENTRY_FUNC
void spm_busy_wait_nse(uint32_t busy_wait_us)
{
	k_busy_wait(busy_wait_us);
}
#endif /* CONFIG_SPM_SERVICE_BUSY_WAIT */
